@@ -6,9 +6,22 @@
 //! * After 8 plots, the colors cycle back and are repeated.
 //!
 
+#[cfg(doctest)]
+mod test_readme {
+    macro_rules! external_doc_test {
+        ($x:expr) => {
+            #[doc = $x]
+            extern "C" {}
+        };
+    }
+
+    external_doc_test!(include_str!("../README.md"));
+}
+
 mod render;
 mod util;
 use std::fmt;
+use tagger::prelude::*;
 
 ///The number of unique colors.
 const NUM_COLORS: usize = 8;
@@ -18,51 +31,9 @@ const WIDTH: f64 = 800.0;
 ///The height of the svg tag.
 const HEIGHT: f64 = 500.0;
 
-///Used internally to implement [`Names`]
-struct NamesStruct<A, B, C, D, E, F> {
-    title: A,
-    xname: B,
-    yname: C,
-    header: D,
-    body: E,
-    footer: F,
-}
-impl<A: Display, B: Display, C: Display, D: Display, E: Display, F: Display> Names
-    for NamesStruct<A, B, C, D, E, F>
-{
-    fn write_header(&self, fm: &mut fmt::Formatter) -> fmt::Result {
-        self.header.fmt(fm)
-    }
-    fn write_body(&self, fm: &mut fmt::Formatter) -> fmt::Result {
-        self.body.fmt(fm)
-    }
-    fn write_footer(&self, fm: &mut fmt::Formatter) -> fmt::Result {
-        self.footer.fmt(fm)
-    }
-    fn write_title(&self, fm: &mut fmt::Formatter) -> fmt::Result {
-        self.title.fmt(fm)
-    }
-    fn write_xname(&self, fm: &mut fmt::Formatter) -> fmt::Result {
-        self.xname.fmt(fm)
-    }
-    fn write_yname(&self, fm: &mut fmt::Formatter) -> fmt::Result {
-        self.yname.fmt(fm)
-    }
-}
-
-///Used internally to write out the header/title/xname/yname.
-trait Names {
-    fn write_header(&self, fm: &mut fmt::Formatter) -> fmt::Result;
-    fn write_body(&self, fm: &mut fmt::Formatter) -> fmt::Result;
-    fn write_footer(&self, fm: &mut fmt::Formatter) -> fmt::Result;
-
-    fn write_title(&self, fm: &mut fmt::Formatter) -> fmt::Result;
-    fn write_xname(&self, fm: &mut fmt::Formatter) -> fmt::Result;
-    fn write_yname(&self, fm: &mut fmt::Formatter) -> fmt::Result;
-}
-
 trait PlotTrait {
     fn write_name(&self, a: &mut fmt::Formatter) -> fmt::Result;
+
     fn iter_first(&mut self) -> &mut dyn Iterator<Item = [f64; 2]>;
     fn iter_second(&mut self) -> &mut dyn Iterator<Item = [f64; 2]>;
 }
@@ -110,6 +81,34 @@ struct Plot<'a> {
     plots: Box<dyn PlotTrait + 'a>,
 }
 
+///
+/// The default svg tag element with the default attributes
+///
+pub fn default_svg<'a>() -> tagger::Element<'a> {
+    elem!("svg", default_svg_attr().build())
+}
+
+///
+/// The default svg tag attributes
+///
+/// They are as follows:
+/// `class="poloto" width="800" height="500" viewBox="0 0 800 500" xmlns="http://www.w3.org/2000/svg"`
+///
+pub fn default_svg_attr<'a>() -> tagger::AttrBuilder<'a> {
+    use tagger::prelude::*;
+
+    let mut k = tagger::attr_builder();
+    k.attr("class", "poloto")
+        .attr("width", DIMENSIONS[0])
+        .attr("height", DIMENSIONS[1])
+        .attr(
+            "viewBox",
+            formatm!("0 0 {} {}", DIMENSIONS[0], DIMENSIONS[1]),
+        )
+        .attr("xmlns", "http://www.w3.org/2000/svg");
+    k
+}
+
 /// Default theme using css variables (with light theme defaults if the variables are not set).
 pub const HTML_CONFIG_CSS_VARIABLE_DEFAULT: &str = "<style>.poloto {\
     stroke-linecap:round;\
@@ -191,13 +190,7 @@ pub const HTML_CONFIG_DARK_DEFAULT: &str = "<style>.poloto {\
     .poloto6fill{fill:lime;}\
     .poloto7fill{fill:chocolate;}</style>";
 
-/// The default SVG Header tag
-pub const SVG_HEADER_DEFAULT: &str = r###"<svg class="poloto" width="800" height="500" viewBox="0 0 800 500" xmlns="http://www.w3.org/2000/svg">"###;
-/// The default SVG Header: attributes only.
-pub const SVG_HEADER_DEFAULT_WITHOUT_TAG: &str = r###"class="poloto" width="800" height="500" viewBox="0 0 800 500" xmlns="http://www.w3.org/2000/svg""###;
-
-/// The default SVG ending tag.
-pub const SVG_FOOTER_DEFAULT: &str = "</svg>";
+const DIMENSIONS: [usize; 2] = [800, 500];
 
 /// Iterators that are passed to the [`Plotter`] plot functions must produce
 /// items that implement this trait.
@@ -265,56 +258,26 @@ impl<T: IntoF64> Plottable for &(T, T) {
     }
 }
 
-/// Shorthand for `plot_with_html_raw(title,xname,yname,SVG_HEADER_DEFAULT,body,SVG_FOOT_DEFAULT);`
-pub fn plot_with_html<'a>(
-    title: impl Display + 'a,
-    xname: impl Display + 'a,
-    yname: impl Display + 'a,
-    body: impl Display + 'a,
-) -> Plotter<'a> {
-    Plotter {
-        names: Box::new(NamesStruct {
-            title,
-            xname,
-            yname,
-            header: SVG_HEADER_DEFAULT,
-            body,
-            footer: SVG_FOOTER_DEFAULT,
-        }),
-        plots: Vec::new(),
-    }
-}
-
-/// Create a [`Plotter`] with the specified title,xname,yname, and custom header,body, and footer.
-/// Consider using some of the default html tags.
-pub fn plot_with_html_raw<'a>(
-    title: impl Display + 'a,
-    xname: impl Display + 'a,
-    yname: impl Display + 'a,
-    header: impl Display + 'a,
-    body: impl Display + 'a,
-    footer: impl Display + 'a,
-) -> Plotter<'a> {
-    Plotter {
-        names: Box::new(NamesStruct {
-            title,
-            xname,
-            yname,
-            header,
-            body,
-            footer,
-        }),
-        plots: Vec::new(),
-    }
-}
-
-/// Shorthand for `plot_with_html(title,xname,yname,HTML_CONFIG_LIGHT_DEFAULT);`
+///
+/// Create a Plotter with preset styling and svg tag.
+///
+/// Shorthand for
+/// ```
+/// use tagger::prelude::*;
+/// let p =poloto::Plotter::new(poloto::default_svg().add(single!(poloto::HTML_CONFIG_LIGHT_DEFAULT)),"title","xname","yname");
+/// ```
+///
 pub fn plot<'a>(
     title: impl Display + 'a,
     xname: impl Display + 'a,
     yname: impl Display + 'a,
 ) -> Plotter<'a> {
-    plot_with_html(title, xname, yname, HTML_CONFIG_LIGHT_DEFAULT)
+    Plotter::new(
+        default_svg().add(single!(HTML_CONFIG_LIGHT_DEFAULT)),
+        title,
+        xname,
+        yname,
+    )
 }
 
 /// Keeps track of plots.
@@ -327,11 +290,35 @@ pub fn plot<'a>(
 /// * The background belongs to the `poloto_background` class.
 ///
 pub struct Plotter<'a> {
-    names: Box<dyn Names + 'a>,
+    element: tagger::Element<'a>,
+    title: Box<dyn fmt::Display + 'a>,
+    xname: Box<dyn fmt::Display + 'a>,
+    yname: Box<dyn fmt::Display + 'a>,
     plots: Vec<Plot<'a>>,
 }
 
 impl<'a> Plotter<'a> {
+    ///
+    /// Create a plotter with the specified element.
+    ///
+    /// ```
+    /// let svg=poloto::default_svg();
+    /// let p=poloto::Plotter::new(svg,"title","x","y");
+    /// ```
+    pub fn new(
+        element: tagger::Element<'a>,
+        title: impl Display + 'a,
+        xname: impl Display + 'a,
+        yname: impl Display + 'a,
+    ) -> Plotter<'a> {
+        Plotter {
+            element,
+            title: Box::new(title),
+            xname: Box::new(xname),
+            yname: Box::new(yname),
+            plots: Vec::new(),
+        }
+    }
     /// Create a line from plots using a SVG polyline element.
     /// The element belongs to the `.poloto[N]stroke` css class.
     ///
@@ -432,27 +419,33 @@ impl<'a> Plotter<'a> {
     }
 
     ///
-    /// Use the plot iterators and generate out a [`RenderResult`] which implements [`std::fmt::Display`]
+    /// Use the plot iterators and generate out a [`tagger::Element`] which implements [`std::fmt::Display`]
+    ///
+    /// Panics if the render fails.
     ///
     /// ```
     /// let data = [[1.0,4.0], [2.0,5.0], [3.0,6.0]];
     /// let mut plotter = poloto::plot("title", "x", "y");
     /// plotter.line("", &data);
-    /// let s = plotter.render().unwrap();
+    /// println!("{}",plotter.render());
+    /// ```
+    pub fn render(&mut self) -> tagger::Element<'a> {
+        render::render(self).unwrap()
+    }
+
+    ///
+    /// Use the plot iterators and generate out a [`tagger::Element`] which implements [`std::fmt::Display`]
+    ///
+    /// Returns a fmt::Error if the render fails.
+    ///
+    /// ```
+    /// let data = [[1.0,4.0], [2.0,5.0], [3.0,6.0]];
+    /// let mut plotter = poloto::plot("title", "x", "y");
+    /// plotter.line("", &data);
+    /// let s = plotter.try_render().unwrap();
     /// println!("{}",s);
     /// ```
-    pub fn render(self) -> Result<RenderResult<'a>, fmt::Error> {
+    pub fn try_render(&mut self) -> Result<tagger::Element<'a>, fmt::Error> {
         render::render(self)
     }
 }
-
-///
-/// A rendered plot graph that implements [`std::fmt::Display`]
-///
-pub struct RenderResult<'a>(tagger::Element<'a>);
-
-impl fmt::Display for RenderResult<'_> {
-    fn fmt(&self, a: &mut fmt::Formatter) -> fmt::Result {
-        self.0.fmt(a)
-    }
-}
