@@ -6,15 +6,20 @@ use std::fmt;
 
 //Returns error if the user supplied format functions don't work.
 //Panics if the element tag writing writes fail
-pub fn render<'b>(plotter: Plotter) -> Result<RenderResult<'b>, fmt::Error> {
-    let Plotter { names, mut plots } = plotter;
+pub fn render<'b>(plotter: &mut Plotter<'b>) -> Result<tagger::Element<'b>, fmt::Error> {
+    let mut element = elem!("");
+    let mut title: Box<dyn fmt::Display> = Box::new("");
+    let mut xname: Box<dyn fmt::Display> = Box::new("");
+    let mut yname: Box<dyn fmt::Display> = Box::new("");
+    let mut plots = Vec::new();
 
-    let header = format!("{}", tagger::moveable_format(|f| names.write_header(f)));
-    let body = format!("{}", tagger::moveable_format(|f| names.write_body(f)));
-    let footer = format!("{}", tagger::moveable_format(|f| names.write_footer(f)));
+    core::mem::swap(&mut element, &mut plotter.element);
+    core::mem::swap(&mut title, &mut plotter.title);
+    core::mem::swap(&mut xname, &mut plotter.xname);
+    core::mem::swap(&mut yname, &mut plotter.yname);
+    core::mem::swap(&mut plots, &mut plotter.plots);
 
-    let mut svg = single!(header);
-    svg.append(single!(body));
+    let mut svg = element;
 
     let width = crate::WIDTH as f64;
     let height = crate::HEIGHT as f64;
@@ -41,7 +46,7 @@ pub fn render<'b>(plotter: Plotter) -> Result<RenderResult<'b>, fmt::Error> {
         m
     } else {
         //TODO test that this looks ok
-        return Ok(RenderResult(svg)); //No plots at all. don't need to draw anything
+        return Ok(svg); //No plots at all. don't need to draw anything
     };
 
     const EPSILON: f64 = f64::MIN_POSITIVE * 10.0;
@@ -219,24 +224,20 @@ pub fn render<'b>(plotter: Plotter) -> Result<RenderResult<'b>, fmt::Error> {
     {
         let spacing = padding / 3.0;
 
-        let name = format!("{}", tagger::moveable_format(|w| { plots.write_name(w) }));
-        let name_exists = !name.is_empty();
-
-        let d = attr_builder()
-            .attr("class", "poloto_text")
-            .attr("alignment-baseline", "middle")
-            .attr("text-anchor", "start")
-            .attr("font-size", "large")
-            .attr("x", width - padding / 1.2)
-            .attr("y", paddingy + (i as f64) * spacing)
-            .build();
-
-        svg.append(elem!("text", d).add(single!(name)));
-
         let legendx1 = width - padding / 1.2 + padding / 30.0;
         let legendy1 = paddingy - padding / 8.0 + (i as f64) * spacing;
 
         //Draw plots
+        let name_exists = {
+            use fmt::Write;
+            let mut wc = WriteCounter::new();
+            write!(
+                wc,
+                "{}",
+                tagger::moveable_format(|w| { plots.write_name(w) })
+            )?;
+            wc.get_counter() != 0
+        };
 
         let it = plots.iter_second().map(|[x, y]| {
             [
@@ -373,11 +374,22 @@ pub fn render<'b>(plotter: Plotter) -> Result<RenderResult<'b>, fmt::Error> {
                 svg.append(elem!("path", d));
             }
         }
-    }
 
-    let title = format!("{}", tagger::moveable_format(|f| names.write_title(f)));
-    let xname = format!("{}", tagger::moveable_format(|f| names.write_xname(f)));
-    let yname = format!("{}", tagger::moveable_format(|f| names.write_yname(f)));
+        //let name = format!("{}", tagger::moveable_format(|w| { plots.write_name(w) }));
+        //let name_exists = !name.is_empty();
+        let name = tagger::moveable_format(move |w| plots.write_name(w));
+
+        let d = attr_builder()
+            .attr("class", "poloto_text")
+            .attr("alignment-baseline", "middle")
+            .attr("text-anchor", "start")
+            .attr("font-size", "large")
+            .attr("x", width - padding / 1.2)
+            .attr("y", paddingy + (i as f64) * spacing)
+            .build();
+
+        svg.append(elem!("text", d).add(single!(name)));
+    }
 
     let d = attr_builder()
         .attr("class", "poloto_text")
@@ -429,6 +441,23 @@ pub fn render<'b>(plotter: Plotter) -> Result<RenderResult<'b>, fmt::Error> {
 
     svg.append(single!("path", d));
 
-    svg.append(single!(footer));
-    Ok(RenderResult(svg))
+    Ok(svg)
+}
+
+struct WriteCounter {
+    counter: usize,
+}
+impl WriteCounter {
+    fn new() -> WriteCounter {
+        WriteCounter { counter: 0 }
+    }
+    fn get_counter(&self) -> usize {
+        self.counter
+    }
+}
+impl fmt::Write for WriteCounter {
+    fn write_str(&mut self, s: &str) -> fmt::Result {
+        self.counter += s.len();
+        Ok(())
+    }
 }
